@@ -21,9 +21,10 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import requests
 
+from neutron._i18n import _LI
+from neutron._i18n import _LW
 from neutron.common import constants as n_const
-from neutron.i18n import _LI
-from neutron.i18n import _LW
+
 
 from networking_arista.common import db_lib
 from networking_arista.common import exceptions as arista_exc
@@ -41,9 +42,11 @@ HEARTBEAT_INTERVAL = 100
 # Commands dict keys
 CMD_SYNC_HEARTBEAT = 'SYNC_HEARTBEAT'
 CMD_REGION_SYNC = 'REGION_SYNC'
+CMD_INSTANCE = 'INSTANCE'
 
 # EAPI error messages of interest
 ERR_CVX_NOT_LEADER = 'only available on cluster leader'
+ERR_DVR_NOT_SUPPORTED = 'EOS version on CVX does not support DVR'
 
 
 class AristaRPCWrapper(object):
@@ -104,16 +107,18 @@ class AristaRPCWrapper(object):
             try:
                 return response.json()['result']
             except KeyError:
-                if (response.json()['error']['code'] == 1002 and
-                    ERR_CVX_NOT_LEADER in
-                   response.json()['error']['data'][0]['errors'][0]):
-                    msg = unicode("%s is not the master" % self._server_ip)
-                    LOG.info(msg)
-                    return None
-                else:
-                    msg = "Unexpected EAPI error"
-                    LOG.info(msg)
-                    raise arista_exc.AristaRpcError(msg=msg)
+                if response.json()['error']['code'] == 1002:
+                    for data in response.json()['error']['data']:
+                        if type(data) == dict and 'errors' in data:
+                            if ERR_CVX_NOT_LEADER in data['errors'][0]:
+                                msg = unicode("%s is not the master" % (
+                                              self._server_ip))
+                                LOG.info(msg)
+                                return None
+
+                msg = "Unexpected EAPI error"
+                LOG.info(msg)
+                raise arista_exc.AristaRpcError(msg=msg)
         except requests.exceptions.ConnectionError:
             msg = (_('Error while trying to connect to %(ip)s') %
                    {'ip': self._server_ip})
@@ -157,6 +162,7 @@ class AristaRPCWrapper(object):
         self.cli_commands['timestamp'] = []
         self.cli_commands[CMD_REGION_SYNC] = ''
         self.cli_commands[CMD_SYNC_HEARTBEAT] = ''
+        self.cli_commands['resource-pool'] = []
 
     def check_cli_commands(self):
         """Checks whether the CLI commands are valid.
@@ -201,17 +207,89 @@ class AristaRPCWrapper(object):
                    'no region %s' % test_region_name]
             self._run_eos_cmds(cmd)
 
+        # Check if the instance command exists
+        instance_command = [
+            'enable',
+            'configure',
+            'cvx',
+            'service openstack',
+            'region %s' % test_region_name,
+            'tenant t1',
+            'instance id i1 type router',
+        ]
+        try:
+            self._run_eos_cmds(instance_command)
+            self.cli_commands[CMD_INSTANCE] = 'instance'
+        except arista_exc.AristaRpcError:
+            self.cli_commands[CMD_INSTANCE] = None
+            LOG.warn(_LW("'instance' command is not available on EOS"))
+        finally:
+            cmd = ['enable', 'configure', 'cvx', 'service openstack',
+                   'no region %s' % test_region_name]
+            self._run_eos_cmds(cmd)
+
+    def check_vlan_type_driver_commands(self):
+        """Checks the validity of CLI commands for Arista's VLAN type driver.
+
+           This method tries to execute the commands used exclusively by the
+           arista_vlan type driver and stores the commands if they succeed.
+        """
+        cmd = ['show openstack resource-pool vlan region %s uuid'
+               % self.region]
+        try:
+            self._run_eos_cmds(cmd)
+            self.cli_commands['resource-pool'] = cmd
+        except arista_exc.AristaRpcError:
+            self.cli_commands['resource-pool'] = []
+            LOG.warn(
+                _LW("'resource-pool' command '%s' is not available on EOS"),
+                cmd)
+
     def _keystone_url(self):
-        keystone_auth_url = ('%s://%s:%s/v2.0/' %
-                             (self.keystone_conf.auth_protocol,
-                              self.keystone_conf.auth_host,
-                              self.keystone_conf.auth_port))
-        return keystone_auth_url
+        if self.keystone_conf.auth_uri:
+            auth_uri = self.keystone_conf.auth_uri.rstrip('/')
+        else:
+            auth_uri = (
+                '%(protocol)s://%(host)s:%(port)s' %
+                {'protocol': self.keystone_conf.auth_protocol,
+                 'host': self.keystone_conf.auth_host,
+                 'port': self.keystone_conf.auth_port})
+        return '%s/v2.0/' % auth_uri
 
     def _heartbeat_required(self, sync, counter=0):
         return (sync and self.cli_commands[CMD_SYNC_HEARTBEAT] and
                 (counter % HEARTBEAT_INTERVAL) == 0)
 
+    def get_vlan_assignment_uuid(self):
+        """Returns the UUID for the region's vlan assignment on CVX
+
+        :returns: string containing the region's vlan assignment UUID
+        """
+        vlan_uuid_cmd = self.cli_commands['resource-pool']
+        if vlan_uuid_cmd:
+            return self._run_eos_cmds(commands=vlan_uuid_cmd)[0]
+        return None
+
+    def get_vlan_allocation(self):
+        """Returns the status of the region's VLAN pool in CVX
+
+        :returns: dictionary containg the assigned, allocated and available
+                  VLANs for the region
+        """
+        if not self.cli_commands['resource-pool']:
+            LOG.warning(_('The version of CVX you are using does not support'
+                          'arista VLAN type driver.'))
+            return None
+        cmd = ['show openstack resource-pools region %s' % self.region]
+        command_output = self._run_eos_cmds(cmd)
+        if command_output:
+            regions = command_output[0]['physicalNetwork']
+            if self.region in regions.keys():
+                return regions[self.region]['vlanPool']['default']
+        return {'assignedVlans': '',
+                'availableVlans': '',
+                'allocatedVlans': ''}
+
     def get_tenants(self):
         """Returns dict of all tenants known by EOS.
 
@@ -224,32 +302,38 @@ class AristaRPCWrapper(object):
 
         return tenants
 
-    def plug_port_into_network(self, vm_id, host_id, port_id,
+    def plug_port_into_network(self, device_id, host_id, port_id,
                                net_id, tenant_id, port_name, device_owner):
         """Generic routine plug a port of a VM instace into network.
 
-        :param vm_id: globally unique identifier for VM instance
-        :param host: ID of the host where the VM is placed
-        :param port_id: globally unique port ID that connects VM to network
+        :param device_id: globally unique identifier for the device
+        :param host: ID of the host where the port is placed
+        :param port_id: globally unique port ID that connects port to network
         :param network_id: globally unique neutron network identifier
         :param tenant_id: globally unique neutron tenant identifier
         :param port_name: Name of the port - for display purposes
         :param device_owner: Device owner - e.g. compute or network:dhcp
         """
         if device_owner == n_const.DEVICE_OWNER_DHCP:
-            self.plug_dhcp_port_into_network(vm_id,
+            self.plug_dhcp_port_into_network(device_id,
                                              host_id,
                                              port_id,
                                              net_id,
                                              tenant_id,
                                              port_name)
         elif device_owner.startswith('compute'):
-            self.plug_host_into_network(vm_id,
+            self.plug_host_into_network(device_id,
                                         host_id,
                                         port_id,
                                         net_id,
                                         tenant_id,
                                         port_name)
+        elif device_owner == n_const.DEVICE_OWNER_DVR_INTERFACE:
+            self.plug_distributed_router_port_into_network(device_id,
+                                                           host_id,
+                                                           port_id,
+                                                           net_id,
+                                                           tenant_id)
 
     def plug_host_into_network(self, vm_id, host, port_id,
                                network_id, tenant_id, port_name):
@@ -293,6 +377,54 @@ class AristaRPCWrapper(object):
                         (dhcp_id, host, port_id))
         self._run_openstack_cmds(cmds)
 
+    def plug_distributed_router_port_into_network(self, router_id, host,
+                                                  port_id, net_id, tenant_id):
+        """Creates a DVR port on EOS.
+
+        :param router_id: globally unique identifier for router instance
+        :param host: ID of the host where the DVR port is placed
+        :param port_id: globally unique port ID that connects port to network
+        :param network_id: globally unique neutron network identifier
+        :param tenant_id: globally unique neutron tenant identifier
+        """
+
+        if not self.cli_commands[CMD_INSTANCE]:
+            LOG.info(ERR_DVR_NOT_SUPPORTED)
+            return
+
+        cmds = ['tenant %s' % tenant_id,
+                'instance id %s type router' % router_id,
+                'port id %s network-id %s hostid %s' % (port_id, net_id, host)]
+        self._run_openstack_cmds(cmds)
+
+    def unplug_port_from_network(self, device_id, device_owner, hostname,
+                                 port_id, network_id, tenant_id):
+        """Removes a port from the device
+
+        :param device_id: globally unique identifier for the device
+        :param host: ID of the host where the device is placed
+        :param port_id: globally unique port ID that connects device to network
+        :param network_id: globally unique neutron network identifier
+        :param tenant_id: globally unique neutron tenant identifier
+        """
+        if device_owner == n_const.DEVICE_OWNER_DHCP:
+            self.unplug_dhcp_port_from_network(device_id,
+                                               hostname,
+                                               port_id,
+                                               network_id,
+                                               tenant_id)
+        elif device_owner.startswith('compute'):
+            self.unplug_host_from_network(device_id,
+                                          hostname,
+                                          port_id,
+                                          network_id,
+                                          tenant_id)
+        elif device_owner == n_const.DEVICE_OWNER_DVR_INTERFACE:
+            self.unplug_distributed_router_port_from_network(device_id,
+                                                             port_id,
+                                                             hostname,
+                                                             tenant_id)
+
     def unplug_host_from_network(self, vm_id, host, port_id,
                                  network_id, tenant_id):
         """Removes previously configured VLAN between TOR and a host.
@@ -325,6 +457,26 @@ class AristaRPCWrapper(object):
                 ]
         self._run_openstack_cmds(cmds)
 
+    def unplug_distributed_router_port_from_network(self, router_id,
+                                                    port_id, host, tenant_id):
+        """Removes a DVR port from EOS.
+
+        :param router_id: globally unique identifier for router instance
+        :param port_id: globally unique port ID that connects port to network
+        :param host: ID of the host where the dhcp is hosted
+        :param tenant_id: globally unique neutron tenant identifier
+        """
+
+        if not self.cli_commands[CMD_INSTANCE]:
+            LOG.info(ERR_DVR_NOT_SUPPORTED)
+            return
+
+        # When the last router port is removed, the router is deleted from EOS.
+        cmds = ['tenant %s' % tenant_id,
+                'instance id %s type router' % router_id,
+                'no port id %s hostid %s' % (port_id, host)]
+        self._run_openstack_cmds(cmds)
+
     def create_network(self, tenant_id, network):
         """Creates a single network on Arista hardware
 
@@ -447,41 +599,76 @@ class AristaRPCWrapper(object):
             cmds.append(self.cli_commands[CMD_SYNC_HEARTBEAT])
         self._run_openstack_cmds(cmds, sync=sync)
 
-    def create_vm_port_bulk(self, tenant_id, vm_port_list, vms, sync=False):
+    def delete_instance_bulk(self, tenant_id, instance_id_list, sync=False):
+        """Deletes instances from EOS for a given tenant
+
+        :param tenant_id : globally unique neutron tenant identifier
+        :param instance_id_list : ids of instances that needs to be deleted.
+        :param sync: This flags indicates that the region is being synced.
+        """
+        cmds = ['tenant %s' % tenant_id]
+        counter = 0
+        for instance in instance_id_list:
+            counter += 1
+            cmds.append('no instance id %s' % instance)
+            if self._heartbeat_required(sync, counter):
+                cmds.append(self.cli_commands[CMD_SYNC_HEARTBEAT])
+
+        if self._heartbeat_required(sync):
+            cmds.append(self.cli_commands[CMD_SYNC_HEARTBEAT])
+        self._run_openstack_cmds(cmds, sync=sync)
+
+    def create_port_bulk(self, tenant_id, neutron_ports, arista_ports,
+                         sync=False):
         """Sends a bulk request to create ports.
 
         :param tenant_id: globaly unique neutron tenant identifier
-        :param vm_port_list: list of ports that need to be created.
-        :param vms: list of vms to which the ports will be attached to.
+        :param neutron_ports: list of neutron ports that need to be created.
+        :param arista_ports: list of ports in the arista db.
         :param sync: This flags indicates that the region is being synced.
         """
         cmds = ['tenant %s' % tenant_id]
         # Create a reference to function to avoid name lookups in the loop
         append_cmd = cmds.append
         counter = 0
-        for port in vm_port_list:
+        for neutron_port in neutron_ports:
             counter += 1
             try:
-                vm = vms[port['device_id']]
+                port = arista_ports[neutron_port['id']]
             except KeyError:
-                LOG.warn(_LW("VM id %(vmid)s not found for port %(portid)s"),
-                         {'vmid': port['device_id'], 'portid': port['id']})
+                LOG.warn(_LW(
+                         "Device id %(device)s not found for port %(port)s"),
+                         {'device': neutron_port['device_id'],
+                          'port': neutron_port['id']})
                 continue
 
-            port_name = '' if 'name' not in port else 'name "%s"' % (
-                port['name']
+            port_name = '' if 'name' not in neutron_port else 'name "%s"' % (
+                neutron_port['name']
             )
 
-            if port['device_owner'] == n_const.DEVICE_OWNER_DHCP:
-                append_cmd('network id %s' % port['network_id'])
+            device_owner = neutron_port['device_owner']
+            if device_owner == n_const.DEVICE_OWNER_DHCP:
+                append_cmd('network id %s' % neutron_port['network_id'])
                 append_cmd('dhcp id %s hostid %s port-id %s %s' %
-                           (vm['vmId'], vm['host'], port['id'], port_name))
-            elif port['device_owner'].startswith('compute'):
-                append_cmd('vm id %s hostid %s' % (vm['vmId'], vm['host']))
+                           (port['deviceId'], port['hosts'][0],
+                            neutron_port['id'], port_name))
+            elif device_owner.startswith('compute'):
+                append_cmd('vm id %s hostid %s' % (port['deviceId'],
+                                                   port['hosts'][0]))
                 append_cmd('port id %s %s network-id %s' %
-                           (port['id'], port_name, port['network_id']))
+                           (neutron_port['id'], port_name,
+                            neutron_port['network_id']))
+            elif device_owner == n_const.DEVICE_OWNER_DVR_INTERFACE:
+                if not self.cli_commands[CMD_INSTANCE]:
+                    LOG.info(ERR_DVR_NOT_SUPPORTED)
+                    continue
+                append_cmd('instance id %s type router' % port['deviceId'])
+                for host in port['hosts']:
+                    append_cmd('port id %s network-id %s hostid %s' % (
+                               neutron_port['id'], neutron_port['network_id'],
+                               host))
             else:
-                LOG.warn(_LW("Unknown device owner: %s"), port['device_owner'])
+                LOG.warn(_LW("Unknown device owner: %s"), device_owner)
                 continue
             if self._heartbeat_required(sync, counter):
                 append_cmd(self.cli_commands[CMD_SYNC_HEARTBEAT])
@@ -829,33 +1016,53 @@ class SyncService(object):
         # In first loop, delete unwanted VM and networks and update networks
         # In second loop, update VMs. This is done to ensure that networks for
         # all tenats are updated before VMs are updated
-        vms_to_update = {}
-        for tenant in db_tenants:
+        instances_to_update = {}
+        for tenant in db_tenants.keys():
             db_nets = db_lib.get_networks(tenant)
-            db_vms = db_lib.get_vms(tenant)
+            db_instances = db_lib.get_vms(tenant)
+
             eos_nets = self._get_eos_networks(eos_tenants, tenant)
             eos_vms = self._get_eos_vms(eos_tenants, tenant)
+            eos_routers = self._get_eos_routers(eos_tenants, tenant)
 
             db_nets_key_set = frozenset(db_nets.keys())
-            db_vms_key_set = frozenset(db_vms.keys())
+            db_instances_key_set = frozenset(db_instances.keys())
             eos_nets_key_set = frozenset(eos_nets.keys())
             eos_vms_key_set = frozenset(eos_vms.keys())
+            eos_routers_key_set = frozenset(eos_routers.keys())
+
+            eos_instances_key_set = (eos_vms_key_set | eos_routers_key_set)
 
             # Find the networks that are present on EOS, but not in Neutron DB
             nets_to_delete = eos_nets_key_set.difference(db_nets_key_set)
 
             # Find the VMs that are present on EOS, but not in Neutron DB
-            vms_to_delete = eos_vms_key_set.difference(db_vms_key_set)
+            instances_to_delete = eos_instances_key_set.difference(
+                db_instances_key_set)
+
+            vms_to_delete = [
+                vm for vm in eos_vms_key_set if vm in instances_to_delete]
+            routers_to_delete = [
+                r for r in eos_routers_key_set if r in instances_to_delete]
 
             # Find the Networks that are present in Neutron DB, but not on EOS
             nets_to_update = db_nets_key_set.difference(eos_nets_key_set)
 
             # Find the VMs that are present in Neutron DB, but not on EOS
-            vms_to_update[tenant] = db_vms_key_set.difference(eos_vms_key_set)
+            instances_to_update[tenant] = db_instances_key_set.difference(
+                eos_instances_key_set)
 
             try:
                 if vms_to_delete:
                     self._rpc.delete_vm_bulk(tenant, vms_to_delete, sync=True)
+                if routers_to_delete:
+                    if self._rpc.cli_commands[CMD_INSTANCE]:
+                        self._rpc.delete_instance_bulk(tenant,
+                                                       routers_to_delete,
+                                                       sync=True)
+                    else:
+                        LOG.info(ERR_DVR_NOT_SUPPORTED)
+
                 if nets_to_delete:
                     self._rpc.delete_network_bulk(tenant, nets_to_delete,
                                                   sync=True)
@@ -878,20 +1085,21 @@ class SyncService(object):
                 self._force_sync = True
 
         # Now update the VMs
-        for tenant in vms_to_update:
-            if not vms_to_update[tenant]:
+        for tenant in instances_to_update:
+            if not instances_to_update[tenant]:
                 continue
             try:
                 # Filter the ports to only the vms that we are interested
                 # in.
-                vm_ports = [
-                    port for port in self._ndb.get_all_ports_for_tenant(
-                        tenant) if port['device_id'] in vms_to_update[tenant]
-                ]
-                if vm_ports:
-                    db_vms = db_lib.get_vms(tenant)
-                    self._rpc.create_vm_port_bulk(tenant, vm_ports, db_vms,
-                                                  sync=True)
+                relevant_ports = []
+                for port in self._ndb.get_all_ports_for_tenant(tenant):
+                    if port['device_id'] in instances_to_update[tenant]:
+                        relevant_ports.append(port)
+
+                if relevant_ports:
+                    db_ports = db_lib.get_ports(tenant)
+                    self._rpc.create_port_bulk(tenant, relevant_ports,
+                                               db_ports, sync=True)
             except arista_exc.AristaRpcError:
                 LOG.warning(EOS_UNREACHABLE_MSG)
                 self._force_sync = True
@@ -946,3 +1154,9 @@ class SyncService(object):
         if eos_tenants and tenant in eos_tenants:
             vms = eos_tenants[tenant]['tenantVmInstances']
         return vms
+
+    def _get_eos_routers(self, eos_tenants, tenant):
+        if eos_tenants and tenant in eos_tenants:
+            if 'tenantRouterInstances' in eos_tenants[tenant]:
+                return eos_tenants[tenant]['tenantRouterInstances']
+        return {}
