@@ -15,6 +15,7 @@
 
 import mock
 
+from neutron.common import constants as n_const
 from neutron.extensions import portbindings
 from neutron.tests.unit import testlib_api
 
@@ -283,7 +284,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.create_port_precommit(port_context)
 
         expected_calls = [
-            mock.call.is_network_provisioned(tenant_id, network_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
             mock.call.remember_tenant(tenant_id),
             mock.call.remember_vm(vm_id, host_id, port_id,
                                   network_id, tenant_id)
@@ -313,7 +314,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.create_port_precommit(port_context)
 
         expected_calls += [
-            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id),
+            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
+                                             None),
             mock.call.remember_tenant(INTERNAL_TENANT_ID),
             mock.call.remember_vm(vm_id, host_id, port_id,
                                   network_id, INTERNAL_TENANT_ID)
@@ -349,7 +351,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.is_port_provisioned(port_id),
-            mock.call.is_network_provisioned(tenant_id, network_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, tenant_id,
                                              port_name, device_owner)
@@ -388,7 +390,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls += [
             mock.call.is_port_provisioned(port_id),
-            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id),
+            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
+                                             None),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, INTERNAL_TENANT_ID,
                                              port_name, device_owner)
@@ -420,7 +423,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         port_id = port_context.current['id']
         expected_calls = [
             mock.call.is_port_provisioned(port_id),
-            mock.call.forget_vm_port(port_id),
+            mock.call.forget_port(port_id, port_context.host),
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -449,7 +452,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         port_id = port_context.current['id']
         expected_calls += [
             mock.call.is_port_provisioned(port_id),
-            mock.call.forget_vm_port(port_id),
+            mock.call.forget_port(port_id, port_context.host),
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -479,9 +482,9 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.delete_port_postcommit(port_context)
 
         expected_calls = [
-            mock.call.is_network_provisioned(tenant_id, network_id),
-            mock.call.unplug_host_from_network(device_id, host_id, port_id,
-                                               network_id, tenant_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.unplug_port_from_network(device_id, 'compute', host_id,
+                                               port_id, network_id, tenant_id),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
             mock.call.forget_tenant(tenant_id),
@@ -514,9 +517,11 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.delete_port_postcommit(port_context)
 
         expected_calls += [
-            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id),
-            mock.call.unplug_host_from_network(device_id, host_id, port_id,
-                                               network_id, INTERNAL_TENANT_ID),
+            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
+                                             None),
+            mock.call.unplug_port_from_network(device_id, 'compute', host_id,
+                                               port_id, network_id,
+                                               INTERNAL_TENANT_ID),
             mock.call.num_nets_provisioned(INTERNAL_TENANT_ID),
             mock.call.num_vms_provisioned(INTERNAL_TENANT_ID),
             mock.call.forget_tenant(INTERNAL_TENANT_ID),
@@ -529,6 +534,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         # Test the case where the port was not provisioned previsouly
         mechanism_arista.db_lib.is_port_provisioned.return_value = False
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
 
         tenant_id = 'ten-1'
         network_id = 'net1-id'
@@ -550,7 +556,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.update_port_precommit(port_context)
 
         expected_calls = [
-            mock.call.is_port_provisioned(port_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.is_port_provisioned(port_id, None),
             mock.call.remember_tenant(tenant_id),
             mock.call.remember_vm(vm_id, host_id, port_id,
                                   network_id, tenant_id)
@@ -578,11 +585,13 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         # Force the check to return port found
         mechanism_arista.db_lib.is_port_provisioned.return_value = True
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
 
         self.drv.update_port_precommit(port_context)
 
         expected_calls += [
-            mock.call.is_port_provisioned(port_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.is_port_provisioned(port_id, None),
             mock.call.update_port(vm_id, host_id, port_id,
                                   network_id, tenant_id)
         ]
@@ -616,7 +625,9 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.update_port_precommit(port_context)
 
         expected_calls += [
-            mock.call.is_port_provisioned(port_id),
+            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
+                                             None),
+            mock.call.is_port_provisioned(port_id, None),
             mock.call.remember_tenant(INTERNAL_TENANT_ID),
             mock.call.remember_vm(vm_id, host_id, port_id,
                                   network_id, INTERNAL_TENANT_ID)
@@ -624,6 +635,37 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
+        router_id = 'r1'
+        # DVR ports
+        # <port, host> does not exist. It should be added to the DB
+        owner = n_const.DEVICE_OWNER_DVR_INTERFACE
+        port_context = self._get_port_context(tenant_id,
+                                              network_id,
+                                              router_id,
+                                              network_context,
+                                              device_owner=owner)
+        mechanism_arista.db_lib.is_port_provisioned.return_value = False
+        self.drv.update_port_precommit(port_context)
+        expected_calls += [
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None),
+            mock.call.is_port_provisioned(port_id, host_id),
+            mock.call.remember_tenant(tenant_id),
+            mock.call.remember_vm(router_id, host_id, port_id,
+                                  network_id, tenant_id)
+        ]
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # Unbind the port. It should be removed from the DB
+        port_context._port['binding:host_id'] = None
+        self.drv.update_port_precommit(port_context)
+        expected_calls += [
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None),
+            mock.call.forget_port(port_id, 'ubuntu1'),
+        ]
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
     def test_update_port_postcommit(self):
         tenant_id = 'ten-1'
         network_id = 'net1-id'
@@ -660,12 +702,13 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         expected_calls = [
             mock.call.NeutronNets(),
             mock.call.get_segmentation_id(tenant_id, network_id),
-            mock.call.is_port_provisioned(port_id),
+            mock.call.is_port_provisioned(port_id, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id),
-            mock.call.is_network_provisioned(tenant_id, network_id),
-            mock.call.unplug_host_from_network(device_id, orig_host_id,
-                                               port_id, network_id, tenant_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.unplug_port_from_network(device_id, 'compute',
+                                               orig_host_id, port_id,
+                                               network_id, tenant_id),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
@@ -712,11 +755,13 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls += [
             mock.call.get_segmentation_id(INTERNAL_TENANT_ID, network_id),
-            mock.call.is_port_provisioned(port_id),
+            mock.call.is_port_provisioned(port_id, None),
             mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
                                              segmentation_id),
-            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id),
-            mock.call.unplug_host_from_network(device_id, orig_host_id,
+            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
+                                             None),
+            mock.call.unplug_port_from_network(device_id, 'compute',
+                                               orig_host_id,
                                                port_id, network_id,
                                                INTERNAL_TENANT_ID),
             mock.call.num_nets_provisioned(INTERNAL_TENANT_ID),
@@ -728,6 +773,396 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
+        # DVR ports
+        tenant_id = 'ten-3'
+        network_id = 'net3-id'
+        segmentation_id = 1003
+        router_id = 'r1'
+
+        network_context = self._get_network_context(tenant_id,
+                                                    network_id,
+                                                    segmentation_id,
+                                                    False)
+        owner = n_const.DEVICE_OWNER_DVR_INTERFACE
+        port_context = self._get_port_context(tenant_id,
+                                              network_id,
+                                              router_id,
+                                              network_context,
+                                              device_owner=owner)
+
+        mechanism_arista.db_lib.is_port_provisioned.return_value = True
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+        mechanism_arista.db_lib.get_shared_network_owner_id.return_value = 1
+        mechanism_arista.db_lib.get_segmentation_id.return_value = 1003
+        mechanism_arista.db_lib.num_nets_provisioned.return_value = 1
+        mechanism_arista.db_lib.num_vms_provisioned.return_value = 1
+
+        # New DVR port - context.original_host is not set and status is ACTIVE
+        #                port should be plugged into the network
+        port = port_context.current
+        device_id = port['device_id']
+        device_owner = port['device_owner']
+        host_id = port['binding:host_id']
+        orig_host_id = 'ubuntu1'
+        port_id = port['id']
+        port_name = port['name']
+
+        self.drv.update_port_postcommit(port_context)
+
+        expected_calls += [
+            mock.call.get_segmentation_id(tenant_id, network_id),
+            mock.call.is_port_provisioned(port_id, port_context.host),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             segmentation_id),
+            mock.call.plug_port_into_network(device_id, host_id, port_id,
+                                             network_id, tenant_id,
+                                             port_name, device_owner)
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # Delete DVR port - context.original is set and the status is DOWN.
+        #                   port should be deleted
+        port_context._status = n_const.PORT_STATUS_DOWN
+        self.drv.update_port_postcommit(port_context)
+
+        expected_calls += [
+            mock.call.get_segmentation_id(tenant_id, network_id),
+            mock.call.is_port_provisioned(port_id, port_context.host),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             segmentation_id),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None),
+            mock.call.unplug_port_from_network(device_id, owner,
+                                               orig_host_id,
+                                               port_id, network_id,
+                                               tenant_id),
+            mock.call.num_nets_provisioned(tenant_id),
+            mock.call.num_vms_provisioned(tenant_id),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+    def test_update_port_precommit_dhcp_reserved_port(self):
+        '''Test to ensure the dhcp port migration is handled correctly.
+
+        Whenever a DHCP agent dies, the port is attached to a dummy device
+        identified by DEVICE_ID_RESERVED_DHCP_PORT. Once the dhcp agent is
+        respawned, the port is reattached to the newly created DHCP instance.
+        This deletes the old dhcp port from the old host and creates the port
+        on the new host. The dhcp port transitions from
+
+        (Active <old host, old dhcp, vif:ovs>) to
+        (Active <old host, reserved, vif:ovs>) to
+        (Down   <new host, new dhcp, vif:unbound>) to
+        (Down   <new host, new dhcp, vif:ovs>) to
+        (Build  <new host, new dhcp, vif:ovs>) to
+        (Active <new host, new dhcp, vif:ovs>)
+
+        When the port is updated to (Active <old host, reserved, vif:ovs>),
+        the port needs to be removed from old host and when the port is updated
+        to (Down <new host, new dhcp, vif:unbound>), it should be created on
+        the new host. Removal and creation should take place in two updates
+        because when the port is updated to
+        (Down <new host, new dhcp, vif:unbound>), the original port would have
+        the device id set to 'reserved_dhcp_port' and so it can't be removed
+        from CVX at that point.
+
+        '''
+
+        tenant_id = 't1'
+        network_id = 'n1'
+        old_device_id = 'old_device_id'
+        new_device_id = 'new_device_id'
+        reserved_device = n_const.DEVICE_ID_RESERVED_DHCP_PORT
+        old_host = 'ubuntu1'
+        new_host = 'ubuntu2'
+        port_id = 101
+        segmentation_id = 1000
+        network_context = self._get_network_context(tenant_id,
+                                                    network_id,
+                                                    segmentation_id,
+                                                    False)
+
+        # (Active <old host, old dhcp, vif:ovs>) to
+        # (Active <old host, reserved, vif:ovs>)
+        context = self._get_port_context(
+            tenant_id, network_id, old_device_id,
+            network_context, device_owner=n_const.DEVICE_OWNER_DHCP)
+        context.current['device_id'] = reserved_device
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+
+        mechanism_arista.db_lib.is_port_provisioned.return_value = True
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_precommit(context)
+
+        expected_calls = [
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.is_port_provisioned(port_id, None),
+            mock.call.update_port(reserved_device,
+                                  old_host, port_id,
+                                  network_id, tenant_id)
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # (Active <old host, reserved, vif:ovs>) to
+        # (Down   <new host, new dhcp, vif:unbound>)
+        context = self._get_port_context(
+            tenant_id, network_id, reserved_device, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP)
+        context.current['device_id'] = new_device_id
+        context.current['binding:host_id'] = new_host
+        context.current['status'] = 'DOWN'
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_precommit(context)
+
+        expected_calls = [
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.is_port_provisioned(port_id, None),
+            mock.call.update_port(new_device_id,
+                                  new_host, port_id,
+                                  network_id, tenant_id)
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # (Down   <new host, new dhcp, vif:unbound>) to
+        # (Down   <new host, new dhcp, vif:ovs>) to
+        context = self._get_port_context(
+            tenant_id, network_id, new_device_id, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP, status='DOWN')
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_precommit(context)
+
+        expected_calls = [
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.is_port_provisioned(port_id, None),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # (Down   <new host, new dhcp, vif:ovs>) to
+        # (Build  <new host, new dhcp, vif:ovs>) to
+        context = self._get_port_context(
+            tenant_id, network_id, new_device_id, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP, status='DOWN')
+
+        context.current['status'] = 'BUILD'
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_precommit(context)
+
+        expected_calls = [
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.is_port_provisioned(port_id, None),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # (Build  <new host, new dhcp, vif:ovs>) to
+        # (Active <new host, new dhcp, vif:ovs>)
+        context = self._get_port_context(
+            tenant_id, network_id, new_device_id, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP, status='BUILD')
+
+        context.current['status'] = 'ACTIVE'
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_precommit(context)
+
+        expected_calls = [
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.is_port_provisioned(port_id, None),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+    def test_update_port_postcommit_dhcp_reserved_port(self):
+        '''Test to ensure the dhcp port migration is handled correctly.
+
+        Whenever a DHCP agent dies, the port is attached to a dummy device
+        identified by DEVICE_ID_RESERVED_DHCP_PORT. Once the dhcp agent is
+        respawned, the port is reattached to the newly created DHCP instance.
+        This deletes the old dhcp port from the old host and creates the port
+        on the new host. The dhcp port transitions from
+
+        (Active <old host, old dhcp, vif:ovs>) to
+        (Active <old host, reserved, vif:ovs>) to
+        (Down   <new host, new dhcp, vif:unbound>) to
+        (Down   <new host, new dhcp, vif:ovs>) to
+        (Build  <new host, new dhcp, vif:ovs>) to
+        (Active <new host, new dhcp, vif:ovs>)
+
+        When the port is updated to (Active <old host, reserved, vif:ovs>),
+        the port needs to be removed from old host and when the port is updated
+        to (Down <new host, new dhcp, vif:unbound>), it should be created on
+        the new host. Removal and creation should take place in two updates
+        because when the port is updated to
+        (Down <new host, new dhcp, vif:unbound>), the original port would have
+        the device id set to 'reserved_dhcp_port' and so it can't be removed
+        from CVX at that point.
+
+        '''
+
+        tenant_id = 't1'
+        network_id = 'n1'
+        old_device_id = 'old_device_id'
+        new_device_id = 'new_device_id'
+        reserved_device = n_const.DEVICE_ID_RESERVED_DHCP_PORT
+        old_host = 'ubuntu1'
+        new_host = 'ubuntu2'
+        port_id = 101
+        seg_id = 1000
+        network_context = self._get_network_context(tenant_id,
+                                                    network_id,
+                                                    seg_id,
+                                                    False)
+
+        # (Active <old host, old dhcp, vif:ovs>) to
+        # (Active <old host, reserved, vif:ovs>)
+        context = self._get_port_context(
+            tenant_id, network_id, old_device_id, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP)
+        context.current['device_id'] = reserved_device
+        port_name = context.current['name']
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+
+        mechanism_arista.db_lib.is_port_provisioned.return_value = True
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+        mechanism_arista.db_lib.get_shared_network_owner_id.return_value = 1
+        mechanism_arista.db_lib.num_nets_provisioned.return_value = 1
+        mechanism_arista.db_lib.num_vms_provisioned.return_value = 1
+        mechanism_arista.db_lib.get_segmentation_id.return_value = seg_id
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_postcommit(context)
+
+        expected_calls = [
+            mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id, seg_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.unplug_port_from_network(old_device_id,
+                                               n_const.DEVICE_OWNER_DHCP,
+                                               old_host,
+                                               port_id, network_id,
+                                               tenant_id),
+            mock.call.num_nets_provisioned(tenant_id),
+            mock.call.num_vms_provisioned(tenant_id),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # (Active <old host, reserved, vif:ovs>) to
+        # (Down   <new host, new dhcp, vif:unbound>)
+        context = self._get_port_context(
+            tenant_id, network_id, reserved_device, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP)
+        context.current['device_id'] = new_device_id
+        context.current['binding:host_id'] = new_host
+        context.current['status'] = 'DOWN'
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_postcommit(context)
+
+        expected_calls = [
+            mock.call.get_segmentation_id('t1', 'n1'),
+            mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id, seg_id),
+            mock.call.is_network_provisioned(tenant_id, network_id, None),
+            mock.call.unplug_port_from_network(reserved_device,
+                                               n_const.DEVICE_OWNER_DHCP,
+                                               old_host,
+                                               port_id, network_id,
+                                               tenant_id),
+            mock.call.num_nets_provisioned(tenant_id),
+            mock.call.num_vms_provisioned(tenant_id),
+            mock.call.plug_port_into_network(new_device_id,
+                                             new_host,
+                                             port_id, network_id,
+                                             tenant_id, port_name,
+                                             n_const.DEVICE_OWNER_DHCP),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # (Down   <new host, new dhcp, vif:unbound>) to
+        # (Down   <new host, new dhcp, vif:ovs>) to
+        context = self._get_port_context(
+            tenant_id, network_id, new_device_id, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP, status='DOWN')
+        context.current['binding:host_id'] = new_host
+        context.original['binding:host_id'] = new_host
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_postcommit(context)
+
+        expected_calls = [
+            mock.call.get_segmentation_id('t1', 'n1'),
+            mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id, seg_id),
+            mock.call.plug_port_into_network(new_device_id,
+                                             new_host,
+                                             port_id, network_id,
+                                             tenant_id, port_name,
+                                             n_const.DEVICE_OWNER_DHCP),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # (Down   <new host, new dhcp, vif:ovs>) to
+        # (Build  <new host, new dhcp, vif:ovs>) to
+        context = self._get_port_context(
+            tenant_id, network_id, new_device_id, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP, status='DOWN')
+        context.current['binding:host_id'] = new_host
+        context.original['binding:host_id'] = new_host
+        context.current['status'] = 'BUILD'
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_postcommit(context)
+
+        expected_calls = [
+            mock.call.get_segmentation_id('t1', 'n1'),
+            mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id, seg_id),
+            mock.call.plug_port_into_network(new_device_id,
+                                             new_host,
+                                             port_id, network_id,
+                                             tenant_id, port_name,
+                                             n_const.DEVICE_OWNER_DHCP),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+        # (Build  <new host, new dhcp, vif:ovs>) to
+        # (Active <new host, new dhcp, vif:ovs>)
+        context = self._get_port_context(
+            tenant_id, network_id, new_device_id, network_context,
+            device_owner=n_const.DEVICE_OWNER_DHCP, status='BUILD')
+        context.current['binding:host_id'] = new_host
+        context.original['binding:host_id'] = new_host
+        context.current['status'] = 'ACTIVE'
+
+        mechanism_arista.db_lib.reset_mock()
+        self.drv.update_port_postcommit(context)
+
+        expected_calls = [
+            mock.call.get_segmentation_id('t1', 'n1'),
+            mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id, seg_id),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
     def _get_network_context(self, tenant_id, net_id, seg_id, shared):
         network = {'id': net_id,
                    'tenant_id': tenant_id,
@@ -738,16 +1173,25 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         return FakeNetworkContext(tenant_id, network, network_segments,
                                   network)
 
-    def _get_port_context(self, tenant_id, net_id, vm_id, network):
-        port = {'device_id': vm_id,
-                'device_owner': 'compute',
+    def _get_port_context(self, tenant_id, net_id, device_id, network,
+                          device_owner='compute', status='ACTIVE'):
+        port = {'device_id': device_id,
+                'device_owner': device_owner,
                 'binding:host_id': 'ubuntu1',
                 'name': 'test-port',
                 'tenant_id': tenant_id,
                 'id': 101,
                 'network_id': net_id
                 }
-        return FakePortContext(port, dict(port), network)
+        orig_port = {'device_id': device_id,
+                     'device_owner': device_owner,
+                     'binding:host_id': 'ubuntu1',
+                     'name': 'test-port',
+                     'tenant_id': tenant_id,
+                     'id': 101,
+                     'network_id': net_id
+                     }
+        return FakePortContext(port, dict(orig_port), network, status)
 
 
 class fake_keystone_info_class(object):
@@ -789,10 +1233,11 @@ class FakeNetworkContext(object):
 class FakePortContext(object):
     """To generate port context for testing purposes only."""
 
-    def __init__(self, port, original_port, network):
+    def __init__(self, port, original_port, network, status):
         self._port = port
         self._original_port = original_port
         self._network_context = network
+        self._status = status
 
     @property
     def current(self):
@@ -814,6 +1259,10 @@ class FakePortContext(object):
     def original_host(self):
         return self._original_port.get(portbindings.HOST_ID)
 
+    @property
+    def status(self):
+        return self._status
+
 
 class FakePluginContext(object):
     """Plugin context for testing purposes only."""
